from apm import *
import matplotlib
import numpy as np
import matplotlib.pyplot as plt

# Select server
s = 'http://xps.apmonitor.com'

# Application name
a = 'trial'



# Clear previous alication
apm(s,a,'clear all')

# Model and Data files
model_file = 'txy.apm'
data_file = 'data.csv'

# Load model file
apm_load(s,a,model_file)

# Option to select solver (1=APOPT, 2=BPOPT, 3=IPOPT)
apm_option(s,a,'nlc.solver',1)
apm_option(s,a,'nlc.max_iter',200)

# Classify parameters and variables
apm_info(s,a,'FV','d12')
apm_info(s,a,'FV','d21')
apm_info(s,a,'MV','p')
apm_info(s,a,'MV','x[1]')
apm_info(s,a,'CV','tc')
apm_info(s,a,'CV','y[1]')

# Load data file
csv_load(s,a,data_file)

# Options
apm_option(s,a,'nlc.csv_read',1)
apm_option(s,a,'nlc.ev_type',1)

# Solve Data reconciliation
apm_option(s,a,'nlc.imode',2)
apm_option(s,a,'d12.status',1)
apm_option(s,a,'d21.status',1)
apm_option(s,a,'tc.fstatus',1)
apm_option(s,a,'y[1].fstatus',1)
output = apm(s,a,'solve')
print(output)

# Display Results in Web Viewer 
#url = apm_web_var(s,a)

# Retrieve results
sol = apm_sol(s,a)

# Read measured values
#data = csv_data(data_file)
import pandas as pd
data = pd.read_csv('data.csv')

#print(Tm)


# (S(theta) - S(theta*)) / S(theta*) <= p / (n-p) * F(p,n-p,1-alpha)
import scipy.stats
p = 2 # number of parameters
n = 16 # number of data points
alpha = 0.05 # alpha, confidence interval
finv = scipy.stats.f.isf(1-alpha,p,(n-p))
rhs = p / (n-p) * finv
best_obj = apm_tag(s,a,'nlc.objfcnval')
ub = best_obj * rhs + best_obj

# Design variables at mesh points
d12_opt = apm_tag(s,a,'d12.newval')
d21_opt = apm_tag(s,a,'d21.newval')
apm_option(s,a,'d12.status',0)
apm_option(s,a,'d21.status',0)
apm_option(s,a,'d12.fstatus',1)
apm_option(s,a,'d21.fstatus',1)
n = 10  # number of points for the meshgrid = n^2
d12 = np.linspace(d12_opt*0.2,d12_opt*2,n)
d21 = np.linspace(d21_opt*0.2,d21_opt*2,n)
d12m, d21m = np.meshgrid(d12, d21)
obj = np.empty_like(d12m)

for i in range(np.size(d12m,0)):
    for j in range(np.size(d12m,1)):
        apm_meas(s,a,'d12',d12m[i,j])    
        apm_meas(s,a,'d21',d21m[i,j])    
        apm(s,a,'solve')
        if (apm_tag(s,a,'nlc.appstatus')==1):
            obj[i,j] = apm_tag(s,a,'nlc.objfcnval')
        else:
            obj[i,j] = np.nan


# Clear previous alication
apm(s,a,'clear all')
# Model and Data files
model_file = 'txy.apm'
# Load model file
apm_load(s,a,model_file)
# Classify parameters and variables

apm_info(s,a,'FV','d12')
apm_info(s,a,'FV','d21')
apm_info(s,a,'MV','p')
apm_info(s,a,'MV','x[1]')
apm_info(s,a,'CV','tc')
apm_info(s,a,'CV','y[1]')

apm_meas(s,a,'d12',d12_opt)
apm_meas(s,a,'d21',d21_opt)
xpred = np.linspace(0,1,30)
ypred = np.empty_like(xpred)
Tpred = np.empty_like(xpred)
i = 0
for xi in xpred:
    apm_meas(s,a,'x[1]',xi)
    apm_option(s,a,'nlc.imode',1)
    output = apm(s,a,'solve')
    ypred[i] = apm_tag(s,a,'y[1].model')
    Tpred[i] = apm_tag(s,a,'tc.model')
    i = i+1

# Display results
plt.figure
plt.scatter(sol['y[1]'],sol['tc'],c='r',marker='x')
plt.scatter(sol['x[1]'],sol['tc'],c='k',marker='x')
plt.scatter(data['x[1]'],data['tc'],c='b',marker='o')
plt.scatter(data['y[1]'],data['tc'],c='y',marker='o')
plt.plot(ypred,Tpred,'r--')
plt.plot(xpred,Tpred,'k--')

plt.legend(['y model','y meas','x meas'])
plt.ylabel('Temperature (degC)')
plt.xlabel('Ethanol Mole Fraction')
plt.axis([0,1,60,75])
plt.savefig('data_fit.png')


# Create a contour plot
# Visit http://matplotlib.org/examples/pylab_examples/contour_demo.html
#   for more examples and options for contour plots
plt.figure()
# Weight contours
CS = plt.contour(d12m, d21m, obj)
plt.clabel(CS, inline=1, fontsize=10)
# Confidence region
CS = plt.contour(d12m, d21m, obj,[ub],colors='k',linewidths=[4.0])
plt.clabel(CS, inline=1, fontsize=10)
# Add some labels
plt.title('Objective Function Contour')
plt.xlabel(r'$\Lambda_{12}$')
plt.ylabel(r'$\Lambda_{21}$')
# Save the figure as a PNG
plt.savefig('contour.png')

print('Lambda 12: ' + str(d12_opt))
print('Lambda 21: ' + str(d21_opt))












plt.show()
